package com.lqkj.link.config;

import io.swagger.v3.oas.models.Components;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.info.License;
import io.swagger.v3.oas.models.security.SecurityRequirement;
import io.swagger.v3.oas.models.security.SecurityScheme;
import org.springdoc.core.models.GroupedOpenApi;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.List;
import java.util.stream.Collectors;

@Configuration
public class OpenApiConfig {

    @Bean
    public OpenAPI customOpenAPI(){
        return new OpenAPI()
                .info(info())
                .externalDocs(externalDocs())
                .components(components())
                .addSecurityItem(securityRequirement());
    }

    private Info info(){
        return new Info()
                .title("灵客空间")
                .version("V1.0.0")
                .description("LINK-SERVER API")
                .license(new License()
                        .name("Apache 2.0") // The Apache License, Version 2.0
                        .url("https://www.apache.org/licenses/LICENSE-2.0.html"))
                .contact(new Contact()
                        .name("灵奇空间")
                        .url("http://www.you07.com")
                        .email("jc.top@qq.com"))
                .termsOfService("http://www.you07.com")
                ;
    }

    private ExternalDocumentation externalDocs() {
        return new ExternalDocumentation()
                .description("灵客空间")
                .url("http://www.you07.com");
    }

    private Components components(){
        return new Components()
                .addSecuritySchemes("Bearer Authorization",
                        new SecurityScheme()
                                .name("Bearer 认证")
                                .type(SecurityScheme.Type.HTTP)
                                .scheme("bearer")
                                .bearerFormat("JWT")
                                .in(SecurityScheme.In.HEADER)
                );

    }

    private SecurityRequirement securityRequirement() {
        return new SecurityRequirement()
                .addList("Bearer Authorization");
    }

    private List<SecurityRequirement> security(Components components) {
        return components.getSecuritySchemes()
                .keySet()
                .stream()
                .map(k -> new SecurityRequirement().addList(k))
                .collect(Collectors.toList());
    }


    /**
     * 通用接口
     * @return
     */
    @Bean
    public GroupedOpenApi publicApi(){
        return GroupedOpenApi.builder()
                .group("通用接口")
                .pathsToMatch(
                        "/jwt/**",
                        "/encrypt/**")
                .build();
    }

    /**
     * 前端接口
     * @return
     */
    @Bean
    public GroupedOpenApi chaserApi(){
        return GroupedOpenApi.builder()
                .group("前端接口")
                .pathsToMatch(
                        "/user/v1/**",
                        "/config/v1/**",
                        "/zone/v1/**",
                        "/geom/v1/**",
                        "/audit/v1/**",
                        "/notice/v1/**",
                        "/layer/v1/**",
                        "/resource/v1/**",
                        "/resource/ossUpload")
                .build();
    }

    /**
     * 管理接口
     * @return
     */
    @Bean
    public GroupedOpenApi supervisorApi(){
        return GroupedOpenApi.builder()
                .group("后端接口")
                .pathsToMatch(
                        "/user/authList",
                        "/zone/template/**",
                        "/resource/manage/**",
                        "/audit/manage/**",
                        "/user/manage/**",
                        "/role/**",
                        "/config/manage/**",
                        "/bulletin/manage/**")
                .build();
    }

}
