package com.lqkj.link.module.authority.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.persistence.*;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "user_info")
@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
public class UserInfo implements UserDetails {
    @Id
    @Column(name = "user_id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Schema(description = "用户ID")
    private Integer userId;
    @Column(name = "user_code")
    @Schema(description = "登录账号")
    private String userCode;
    @Column(name = "display_name")
    @Schema(description = "显示名称")
    private String displayName;
    @Column(name = "password")
    @Schema(description = "密码")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private String password;
    @Column(name = "authorization_code")
    @Schema(description = "授权码")
    private String authorizationCode;
    @Column(name = "head_img")
    @Schema(description = "头像图片地址")
    private String headImg;
    @Column(name = "file_save_path")
    @Schema(description = "文件存储路径")
    private String fileSavePath;
    @Transient
    @Schema(description = "是否有审核权限")
    private Boolean hasAuth;
    @Column(name = "locking")
    @Schema(description = "是否锁定")
    @JsonIgnore
    private Boolean locking;
    @Column(name = "login_error_count")
    @Schema(description = "登录错误次数")
    @JsonIgnore
    private Integer loginErrorCount;
    @Column(name = "lock_time")
    @Schema(description = "锁定时间")
    @JsonIgnore
    @JsonFormat(pattern = "YYYY-MM-dd HH:mm:ss", timezone = "Asia/Shanghai")
    private Date lockTime;
    @Column(name = "update_time")
    @JsonFormat(pattern = "YYYY-MM-dd HH:mm:ss", timezone = "Asia/Shanghai")
    @Schema(description = "更新时间")
    private Date updateTime;
    @Column(name = "has_manage")
    @Schema(description = "是否能进入后台")
    private Boolean hasManage;
    @Transient
    @Schema(description = "角色列表")
    private List<RoleInfo> roleInfoList;
    @Transient
    @Schema(description = "角色ID数组")
    private List<Integer> roleIds;
    @Transient
    private String roleNames;
    @Column(name = "refresh_resource")
    private Boolean refreshResource;

    @Override
    @JsonIgnore
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return null;
    }

    @Override
    @JsonIgnore
    public String getUsername() {
        return userCode;
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isEnabled() {
        return true;
    }
}
