package com.lqkj.link.util;

import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

/**
 * @ClassName FileUtils
 * @Description: TODO
 * @Author wells
 * @Date 2019/11/29 9:22
 * @Version V1.0
 **/
public class FileUtils {

    public static void dirCopy(File src, String destPath) throws Exception {
        if (!new File(destPath).exists()) {
            new File(destPath).mkdirs();
        }
        for (File s : src.listFiles()) {
            if (s.isFile()) {
                fileCopy(s.getPath(), destPath + File.separator + s.getName());
            } else {
                dirCopy(s, destPath + File.separator + s.getName());
            }
        }
    }

    public static void fileCopy(String srcPath, String destPath) throws IOException {
        File src = new File(srcPath);
        File dest = new File(destPath);
        try (InputStream is = new BufferedInputStream(new FileInputStream(src));
             OutputStream out = new BufferedOutputStream(new FileOutputStream(dest))) {
            byte[] flush = new byte[1024];
            int len = -1;
            while ((len = is.read(flush)) != -1) {
                out.write(flush, 0, len);
            }
            out.flush();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void fileCopyIn(InputStream srcIn, String destPath, String fileName) {
        if (!new File(destPath).exists()) {
            new File(destPath).mkdirs();
        }
        File dest = new File(destPath + fileName);
        try (
                OutputStream out = new BufferedOutputStream(new FileOutputStream(dest))) {
            byte[] flush = new byte[1024];
            int len = -1;
            while ((len = srcIn.read(flush)) != -1) {
                out.write(flush, 0, len);
            }
            out.flush();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 将base64字符解码保存文件
     *
     * @param base64Code
     * @param targetPath
     * @throws Exception
     */

    public static void decoderBase64File(String base64Code, String targetPath)
            throws Exception {
        File file = new File(targetPath.substring(0, targetPath.lastIndexOf("/")));
        if (!file.exists()) {
            file.mkdirs();
        }
        byte[] buffer = Base64.getDecoder().decode(base64Code);
        FileOutputStream out = new FileOutputStream(targetPath);
        out.write(buffer);
        out.close();
    }

    /**
     * 判断是否为图片格式
     *
     * @param fileExtension
     * @return
     */
    public static Boolean isImgFile(String fileExtension) {
        if (fileExtension.toLowerCase().contains("jpg") || fileExtension.toLowerCase().contains("jpeg") || fileExtension.toLowerCase().contains("png")
                || fileExtension.contains("gif")) {
            return true;
        }
        return false;
    }

    public static void downloadFile(String url, String targetPath) throws Exception {
        File file = new File(targetPath.substring(0, targetPath.lastIndexOf("/")));
        if (!file.exists()) {
            file.mkdirs();
        }

        URL downloadUrl = new URL(url);
        // 下载网络文件
        int bytesum = 0;
        int byteread = 0;
        try {
            URLConnection conn = downloadUrl.openConnection();
            InputStream inStream = conn.getInputStream();
            FileOutputStream fs = new FileOutputStream(targetPath);

            byte[] buffer = new byte[1024];
            while ((byteread = inStream.read(buffer)) != -1) {
                bytesum += byteread;
                System.out.println(bytesum);
                fs.write(buffer, 0, byteread);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 保存上传的文件到本地
     *
     * @param file 被删除文件的文件名
     * @param path 保存路径
     * @return 保存后的文件
     */
    public static File saveFile(MultipartFile file, String path, String name) {
        try (InputStream inputStream = file.getInputStream()) {
            File writeFile = new File(path + name);
            org.apache.commons.io.FileUtils.copyInputStreamToFile(inputStream, writeFile);
            return writeFile;
        } catch (Exception e) {
            return null;
        }
    }

    public static List<File> searchFiles(File folder, final String keyword) {
        List<File> result = new ArrayList<File>();
        if (folder.isFile())
            result.add(folder);

        File[] subFolders = folder.listFiles(new FileFilter() {
            @Override
            public boolean accept(File file) {
                if (file.isDirectory()) {
                    return true;
                }
                if (file.getName().toLowerCase().contains(keyword)) {
                    return true;
                }
                return false;
            }
        });

        if (subFolders != null) {
            for (File file : subFolders) {
                if (file.isFile()) {
                    // 如果是文件则将文件添加到结果列表中
                    result.add(file);
                } else {
                    // 如果是文件夹，则递归调用本方法，然后把所有的文件加到结果列表中
                    result.addAll(searchFiles(file, keyword));
                }
            }
        }
        return result;
    }

    public static long getFolderSize(File folder) {
        long totalSize = 0;

        // 检查路径是否存在且为目录
        if (folder.exists() && folder.isDirectory()) {
            File[] files = folder.listFiles();
            if (files != null) { // 处理可能为 null 的情况
                for (File file : files) {
                    if (file.isFile()) {
                        // 累加文件大小
                        totalSize += file.length();
                    } else if (file.isDirectory()) {
                        // 递归计算子目录大小
                        totalSize += getFolderSize(file);
                    }
                }
            }
        } else {
            System.out.println("The specified path does not exist or is not a directory.");
        }

        return totalSize;
    }

    public static String convertBytes(long bytes) {
        if (bytes < 0) {
            return "Invalid size";
        }

        // 定义单位
        String[] units = {"B", "KB", "MB", "GB"};
        int unitIndex = 0;

        // 逐步转换为更大的单位
        double size = (double) bytes;

        while (unitIndex < units.length - 1) {
            size /= 1024;
            unitIndex++;
        }

        // 使用 String.format 格式化输出
        return String.format("%.2f%s", size, units[unitIndex]);
    }
}
